@@ -1,17 +1,17 @@
-from collections import namedtuple
-
 import numpy as np
 
 from . import lib, utils
+from .dtypes import _INDEX
 from .utils import _CArray, output_type
 
 
 class _AllIndices:
-    __slots__ = "_carg", "name"
+    __slots__ = "_carg", "name", "_expr_name"
 
     def __init__(self):
         self._carg = lib.GrB_ALL
         self.name = "GrB_ALL"
+        self._expr_name = ":"
 
     def __reduce__(self):
         return "_ALL_INDICES"
@@ -19,35 +19,128 @@ class _AllIndices:
 
 _ALL_INDICES = _AllIndices()
 
-# We may want this as a class eventually (different repr, etc.)
-AxisIndex = namedtuple("AxisIndex", ["size", "index", "cscalar"])
+
+class AxisIndex:
+    __slots__ = "size", "index", "cscalar", "dimsize"
+
+    def __init__(self, size, index, cscalar, dimsize):
+        self.size = size
+        self.index = index
+        self.cscalar = cscalar
+        self.dimsize = dimsize
+
+    @property
+    def _carg(self):
+        return self.index._carg
+
+    @property
+    def name(self):
+        return self.index.name
+
+    @property
+    def _expr_name(self):
+        if self.size is None:
+            return f"{self.index.value}"
+        idx = self._py_index()
+        if type(idx) is slice:
+            rv = f"{'' if idx.start is None else idx.start}:{'' if idx.stop is None else idx.stop}"
+            if idx.step is not None:
+                return f"{rv}:{idx.step}"
+            return rv
+        if idx.size < 6:
+            return f"[{', '.join(map(str, idx))}]"
+        else:
+            return f"[{', '.join(map(str, idx[:3]))}, ...]"
+
+    def _py_index(self):
+        """Convert resolved index back into a valid Python index"""
+        if self.size is None:
+            return self.index.value
+        if self.index is _ALL_INDICES:
+            return slice(None)
+        from ._slice import gxb_backwards, gxb_range, gxb_stride
+
+        if self.cscalar is gxb_backwards:
+            start, stop, step = self.index.array.tolist()
+            size = self.dimsize
+            stop -= size + 1
+            step = -step
+        elif self.cscalar is gxb_range:
+            start, stop = self.index.array.tolist()
+            step = None
+            stop += 1
+        elif self.cscalar is gxb_stride:
+            start, stop, step = self.index.array.tolist()
+            stop += 1
+        else:
+            return self.index.array
+        if (
+            start == 0
+            and (step is None or step > 0)
+            or start == self.dimsize - 1
+            and step is not None
+            and step < 0
+        ):
+            start = None
+        if (
+            stop == self.dimsize
+            and (step is None or step > 0)
+            or stop == -self.dimsize - 1
+            and step is not None
+            and step < 0
+        ):
+            stop = None
+        return slice(start, stop, step)
 
 
 class IndexerResolver:
-    __slots__ = "obj", "indices", "__weakref__"
+    __slots__ = "obj", "indices", "shape", "__weakref__"
 
     def __init__(self, obj, indices):
         self.obj = obj
         if indices is Ellipsis:
-            from .scalar import _CScalar
+            from .scalar import _as_scalar
             from .vector import Vector
 
             if type(obj) is Vector:
-                self.indices = [AxisIndex(obj._size, _ALL_INDICES, _CScalar(obj._size))]
+                self.indices = [
+                    AxisIndex(
+                        obj._size,
+                        _ALL_INDICES,
+                        _as_scalar(obj._size, _INDEX, is_cscalar=True),
+                        obj._size,
+                    )
+                ]
+                self.shape = (obj._size,)
             else:
                 self.indices = [
-                    AxisIndex(obj._nrows, _ALL_INDICES, _CScalar(obj._nrows)),
-                    AxisIndex(obj._ncols, _ALL_INDICES, _CScalar(obj._ncols)),
+                    AxisIndex(
+                        obj._nrows,
+                        _ALL_INDICES,
+                        _as_scalar(obj._nrows, _INDEX, is_cscalar=True),
+                        obj._nrows,
+                    ),
+                    AxisIndex(
+                        obj._ncols,
+                        _ALL_INDICES,
+                        _as_scalar(obj._ncols, _INDEX, is_cscalar=True),
+                        obj._ncols,
+                    ),
                 ]
+                self.shape = (obj._nrows, obj._ncols)
         else:
             self.indices = self.parse_indices(indices, obj.shape)
+            self.shape = tuple(index.size for index in self.indices if index.size is not None)
 
     @property
     def is_single_element(self):
-        for idx in self.indices:
-            if idx.size is not None:
-                return False
-        return True
+        return not self.shape
+
+    @property
+    def py_indices(self):
+        if self.obj.ndim > 1:
+            return tuple(index._py_index() for index in self.indices)
+        return self.indices[0]._py_index()
 
     def parse_indices(self, indices, shape):
         """
@@ -77,12 +170,16 @@ class IndexerResolver:
         return out
 
     def parse_index(self, index, typ, size):
-        from .scalar import _CScalar
+        from .scalar import _as_scalar
 
         if np.issubdtype(typ, np.integer):
             if index >= size:
                 raise IndexError(f"Index out of range: index={index}, size={size}")
-            return AxisIndex(None, _CScalar(int(index)), None)
+            if np.issubdtype(typ, np.signedinteger) and index < 0:
+                index = index + size
+                if index < 0:
+                    raise IndexError(f"Index out of range: index={index - size}, size={size}")
+            return AxisIndex(None, _as_scalar(int(index), _INDEX, is_cscalar=True), None, size)
         if typ is list:
             pass
         elif typ is slice:
@@ -95,14 +192,28 @@ class IndexerResolver:
                 raise TypeError(f"Invalid number of dimensions for index: {len(index.shape)}")
             if not np.issubdtype(index.dtype, np.integer):
                 raise TypeError(f"Invalid dtype for index: {index.dtype}")
-            return AxisIndex(len(index), _CArray(index), _CScalar(len(index)))
+            if np.issubdtype(index.dtype, np.signedinteger):
+                is_negative = index < 0
+                if is_negative.any():
+                    index = np.where(is_negative, index + size, index)
+                    if (index < 0).any():
+                        bad_index = index[index < 0][0] - size
+                        raise IndexError(f"Index out of range: index={bad_index}, size={size}")
+            return AxisIndex(
+                len(index), _CArray(index), _as_scalar(len(index), _INDEX, is_cscalar=True), size
+            )
         else:
             from .scalar import Scalar
 
             if typ is Scalar:
-                if index.dtype.name.startswith("F"):
+                if not np.issubdtype(index.dtype.np_type, np.integer):
                     raise TypeError(f"An integer is required for indexing.  Got: {index.dtype}")
-                return AxisIndex(None, _CScalar(index), None)
+                value = int(index)
+                if np.issubdtype(index.dtype.np_type, np.signedinteger) and value < 0:
+                    value = value + size
+                    if value < 0:
+                        raise IndexError(f"Index out of range: index={value - size}, size={size}")
+                return AxisIndex(None, _as_scalar(value, _INDEX, is_cscalar=True), None, size)
 
             from .matrix import Matrix, TransposedMatrix
             from .vector import Vector
@@ -131,7 +242,7 @@ class IndexerResolver:
                         f"`x(mask={index.name}) << value`."
                     )
                 raise TypeError(f"Invalid type for index: {typ}; unable to convert to list")
-        return AxisIndex(len(index), _CArray(index), _CScalar(len(index)))
+        return self.parse_index(np.array(index), np.ndarray, size)
 
     def get_index(self, dim):
         """Return a new IndexerResolver with index for the selected dimension"""
@@ -169,11 +280,13 @@ class Assigner:
 
 
 class AmbiguousAssignOrExtract:
-    __slots__ = "parent", "resolved_indexes", "__weakref__"
+    __slots__ = "parent", "resolved_indexes", "_value", "__weakref__"
+    _is_scalar = False
 
     def __init__(self, parent, resolved_indexes):
         self.parent = parent
         self.resolved_indexes = resolved_indexes
+        self._value = None
 
     def __call__(self, *args, **kwargs):
         # Occurs when user calls `C[index](params)`
@@ -192,57 +305,22 @@ class AmbiguousAssignOrExtract:
             raise TypeError("'TransposedMatrix' object does not support item assignment")
         Updater(self.parent)._setitem(self.resolved_indexes, obj, is_submask=False)
 
-    @property
-    def value(self):
-        if not self.resolved_indexes.is_single_element:
-            raise AttributeError("Only Scalars have `.value` attribute")
-        scalar = self.parent._extract_element(self.resolved_indexes, name="s_extract")
-        return scalar.value
-
     def new(self, dtype=None, *, mask=None, input_mask=None, name=None):
         """
         Force extraction of the indexes into a new object
         dtype and mask are the only controllable parameters.
         """
-        if self.resolved_indexes.is_single_element:
-            if mask is not None or input_mask is not None:
-                raise TypeError("mask is not allowed for single element extraction")
-            return self.parent._extract_element(self.resolved_indexes, dtype, name=name)
-        else:
-            if input_mask is not None:
-                if mask is not None:
-                    raise TypeError("mask and input_mask arguments cannot both be given")
-                from .base import _check_mask
+        if input_mask is not None:
+            if mask is not None:
+                raise TypeError("mask and input_mask arguments cannot both be given")
+            from .base import _check_mask
 
-                _check_mask(input_mask, output=self.parent)
-                mask = self._input_mask_to_mask(input_mask)
-            delayed_extractor = self.parent._prep_for_extract(self.resolved_indexes)
-            return delayed_extractor.new(dtype, mask=mask, name=name)
+            _check_mask(input_mask, output=self.parent)
+            mask = self._input_mask_to_mask(input_mask)
+        delayed_extractor = self.parent._prep_for_extract(self.resolved_indexes)
+        return delayed_extractor.new(dtype, mask=mask, name=name)
 
-    def __eq__(self, other):
-        if not self.resolved_indexes.is_single_element:
-            raise TypeError(
-                f"__eq__ not defined for objects of type {type(self)}.  "
-                f"Use `.new()` to create a new object, then use `.isequal` method."
-            )
-        return self.value == other
-
-    def __bool__(self):
-        if not self.resolved_indexes.is_single_element:
-            raise TypeError(f"__bool__ not defined for objects of type {type(self)}.")
-        return bool(self.value)
-
-    def __float__(self):
-        if not self.resolved_indexes.is_single_element:
-            raise TypeError(f"__float__ not defined for objects of type {type(self)}.")
-        return float(self.value)
-
-    def __int__(self):
-        if not self.resolved_indexes.is_single_element:
-            raise TypeError(f"__int__ not defined for objects of type {type(self)}.")
-        return int(self.value)
-
-    __index__ = __int__
+    dup = new
 
     def _extract_delayed(self):
         """Return an Expression object, treating this as an extract call"""
@@ -294,6 +372,20 @@ class AmbiguousAssignOrExtract:
         mask_value = mask_expr.new(name="mask_temp")
         return type(input_mask)(mask_value)
 
+    def _repr_html_(self):
+        from . import formatting
+
+        return formatting.format_index_expression_html(self)
+
+    def __repr__(self):
+        from . import formatting
+
+        return formatting.format_index_expression(self)
+
+    @property
+    def dtype(self):
+        return self.parent.dtype
+
 
 class Updater:
     __slots__ = "parent", "kwargs", "__weakref__"
@@ -336,7 +428,13 @@ class Updater:
         if resolved_indexes.is_single_element:
             self.parent._delete_element(resolved_indexes)
         else:
-            raise TypeError("Remove Element only supports a single index")
+            # Delete selection by assigning an empty scalar
+            from .scalar import Scalar
+
+            scalar = Scalar.new(
+                self.parent.dtype, is_cscalar=False, name="s_empty"  # pragma: is_grbscalar
+            )
+            self._setitem(resolved_indexes, scalar, is_submask=False)
 
     def __lshift__(self, expr):
         # Occurs when user calls `C(params) << expr`
