@@ -11,7 +11,7 @@ from .exceptions import NoValue, check_status
 from .expr import AmbiguousAssignOrExtract, IndexerResolver, Updater
 from .mask import StructuralMask, ValueMask
 from .operator import get_typed_op
-from .scalar import Scalar, ScalarExpression, _CScalar
+from .scalar import _MATERIALIZE, Scalar, ScalarExpression, _as_scalar
 from .utils import (
     _CArray,
     _Pointer,
@@ -101,25 +101,42 @@ class Matrix(BaseType):
 
     def __getitem__(self, keys):
         resolved_indexes = IndexerResolver(self, keys)
-        return AmbiguousAssignOrExtract(self, resolved_indexes)
+        shape = resolved_indexes.shape
+        if not shape:
+            from .scalar import ScalarIndexExpr
+
+            return ScalarIndexExpr(self, resolved_indexes)
+        elif len(shape) == 1:
+            from .vector import VectorIndexExpr
+
+            return VectorIndexExpr(self, resolved_indexes, *shape)
+        else:
+            return MatrixIndexExpr(self, resolved_indexes, *shape)
 
     def __setitem__(self, keys, expr):
         Updater(self)[keys] = expr
 
     def __contains__(self, index):
         extractor = self[index]
-        if not extractor.resolved_indexes.is_single_element:
+        if not extractor._is_scalar:
             raise TypeError(
                 f"Invalid index to Matrix contains: {index!r}.  A 2-tuple of ints is expected.  "
                 "Doing `(i, j) in my_matrix` checks whether a value is present at that index."
             )
         scalar = extractor.new(name="s_contains")
-        return not scalar.is_empty
+        return not scalar._is_empty
 
     def __iter__(self):
+        self.wait()  # sort in SS
         rows, columns, values = self.to_values()
         return zip(rows.flat, columns.flat)
 
+    def __sizeof__(self):
+        size = ffi_new("size_t*")
+        scalar = Scalar(size, _INDEX, name="s_size", is_cscalar=True, empty=True)
+        call("GxB_Matrix_memoryUsage", [_Pointer(scalar), self])
+        return size[0] + object.__sizeof__(self)
+
     def isequal(self, other, *, check_dtype=False):
         """
         Check for exact equality (same size, same empty values)
@@ -149,7 +166,7 @@ class Matrix(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce_scalar(monoid.land).new().value
+        return matches.reduce_scalar(monoid.land, allow_empty=False).new().value
 
     def isclose(self, other, *, rel_tol=1e-7, abs_tol=0.0, check_dtype=False):
         """
@@ -177,19 +194,19 @@ class Matrix(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce_scalar(monoid.land).new().value
+        return matches.reduce_scalar(monoid.land, allow_empty=False).new().value
 
     @property
     def nrows(self):
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_nrows", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_nrows", is_cscalar=True, empty=True)
         call("GrB_Matrix_nrows", [_Pointer(scalar), self])
         return n[0]
 
     @property
     def ncols(self):
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_ncols", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_ncols", is_cscalar=True, empty=True)
         call("GrB_Matrix_ncols", [_Pointer(scalar), self])
         return n[0]
 
@@ -200,7 +217,7 @@ class Matrix(BaseType):
     @property
     def nvals(self):
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_nvals", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_nvals", is_cscalar=True, empty=True)
         call("GrB_Matrix_nvals", [_Pointer(scalar), self])
         return n[0]
 
@@ -219,11 +236,11 @@ class Matrix(BaseType):
         call("GrB_Matrix_clear", [self])
 
     def resize(self, nrows, ncols):
-        nrows = _CScalar(nrows)
-        ncols = _CScalar(ncols)
+        nrows = _as_scalar(nrows, _INDEX, is_cscalar=True)
+        ncols = _as_scalar(ncols, _INDEX, is_cscalar=True)
         call("GrB_Matrix_resize", [self, nrows, ncols])
-        self._nrows = nrows.scalar.value
-        self._ncols = ncols.scalar.value
+        self._nrows = nrows.value
+        self._ncols = ncols.value
 
     def to_values(self, dtype=None):
         """
@@ -235,7 +252,7 @@ class Matrix(BaseType):
         columns = _CArray(size=nvals, name="&columns_array")
         values = _CArray(size=nvals, dtype=self.dtype, name="&values_array")
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_nvals", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_nvals", is_cscalar=True, empty=True)
         scalar.value = nvals
         call(
             f"GrB_Matrix_extractTuples_{self.dtype.name}",
@@ -288,7 +305,7 @@ class Matrix(BaseType):
         values = _CArray(values, self.dtype)
         call(
             f"GrB_Matrix_build_{self.dtype.name}",
-            [self, rows, columns, values, _CScalar(n), dup_op],
+            [self, rows, columns, values, _as_scalar(n, _INDEX, is_cscalar=True), dup_op],
         )
         # Check for duplicates when dup_op was not provided
         if not dup_op_given and self._nvals < n:
@@ -321,10 +338,7 @@ class Matrix(BaseType):
         and make it safe to use as input parameters on multiple threads.
         """
         # TODO: expose COMPLETE or MATERIALIZE options to the user
-        call(
-            "GrB_Matrix_wait",
-            [self, Scalar.from_value(lib.GrB_MATERIALIZE, name="GrB_MATERIALIZE")],
-        )
+        call("GrB_Matrix_wait", [self, _MATERIALIZE])
 
     @classmethod
     def new(cls, dtype, nrows=0, ncols=0, *, name=None):
@@ -335,11 +349,11 @@ class Matrix(BaseType):
         new_matrix = ffi_new("GrB_Matrix*")
         dtype = lookup_dtype(dtype)
         rv = cls(new_matrix, dtype, name=name)
-        nrows = _CScalar(nrows)
-        ncols = _CScalar(ncols)
+        nrows = _as_scalar(nrows, _INDEX, is_cscalar=True)
+        ncols = _as_scalar(ncols, _INDEX, is_cscalar=True)
         call("GrB_Matrix_new", [_Pointer(rv), dtype, nrows, ncols])
-        rv._nrows = nrows.scalar.value
-        rv._ncols = ncols.scalar.value
+        rv._nrows = nrows.value
+        rv._ncols = ncols.value
         return rv
 
     @classmethod
@@ -480,6 +494,70 @@ class Matrix(BaseType):
             expr.new(name="")  # incompatible shape; raise now
         return expr
 
+    def ewise_union(self, other, op, left_default, right_default):
+        """
+        GxB_Matrix_eWiseUnion
+
+        This is similar to `ewise_add` in that result will contain the union of
+        indices from both Matrices.  Unlike `ewise_add`, this will use
+        ``left_default`` for the left value when there is a value on the right
+        but not the left, and ``right_default`` for the right value when there
+        is a value on the left but not the right.
+
+        ``op`` should be a BinaryOp or Monoid.
+        """
+        # SS, SuiteSparse-specific: eWiseUnion
+        method_name = "ewise_union"
+        other = self._expect_type(other, Matrix, within=method_name, argname="other", op=op)
+        if type(left_default) is not Scalar:
+            try:
+                left = Scalar.from_value(
+                    left_default, is_cscalar=False, name=""  # pragma: is_grbscalar
+                )
+            except TypeError:
+                left = self._expect_type(
+                    left_default,
+                    Scalar,
+                    within=method_name,
+                    keyword_name="left_default",
+                    extra_message="Literal scalars also accepted.",
+                    op=op,
+                )
+        else:
+            left = _as_scalar(left_default, is_cscalar=False)  # pragma: is_grbscalar
+        if type(right_default) is not Scalar:
+            try:
+                right = Scalar.from_value(
+                    right_default, is_cscalar=False, name=""  # pragma: is_grbscalar
+                )
+            except TypeError:
+                right = self._expect_type(
+                    right_default,
+                    Scalar,
+                    within=method_name,
+                    keyword_name="right_default",
+                    extra_message="Literal scalars also accepted.",
+                    op=op,
+                )
+        else:
+            right = _as_scalar(right_default, is_cscalar=False)  # pragma: is_grbscalar
+        scalar_dtype = unify(left.dtype, right.dtype)
+        nonscalar_dtype = unify(self.dtype, other.dtype)
+        op = get_typed_op(op, scalar_dtype, nonscalar_dtype, is_left_scalar=True, kind="binary")
+        self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
+        if op.opclass == "Monoid":
+            op = op.binaryop
+        expr = MatrixExpression(
+            method_name,
+            "GxB_Matrix_eWiseUnion",
+            [self, left, other, right],
+            op=op,
+            expr_repr="{0.name}.{method_name}({2.name}, {op}, {1._expr_name}, {3._expr_name})",
+        )
+        if self.shape != other.shape:
+            expr.new(name="")  # incompatible shape; raise now
+        return expr
+
     def mxv(self, other, op=semiring.plus_times):
         """
         GrB_mxv
@@ -578,7 +656,7 @@ class Matrix(BaseType):
         elif right is None:
             if type(left) is not Scalar:
                 try:
-                    left = Scalar.from_value(left, name="")
+                    left = Scalar.from_value(left, is_cscalar=None, name="")
                 except TypeError:
                     left = self._expect_type(
                         left,
@@ -599,13 +677,16 @@ class Matrix(BaseType):
                     argname="op",
                     extra_message=extra_message,
                 )
-            cfunc_name = f"GrB_Matrix_apply_BinaryOp1st_{left.dtype}"
-            args = [_CScalar(left), self]
-            expr_repr = "{1.name}.apply({op}, left={0})"
+            if left._is_cscalar:
+                cfunc_name = f"GrB_Matrix_apply_BinaryOp1st_{left.dtype}"
+            else:
+                cfunc_name = "GrB_Matrix_apply_BinaryOp1st_Scalar"
+            args = [left, self]
+            expr_repr = "{1.name}.apply({op}, left={0._expr_name})"
         elif left is None:
             if type(right) is not Scalar:
                 try:
-                    right = Scalar.from_value(right, name="")
+                    right = Scalar.from_value(right, is_cscalar=None, name="")
                 except TypeError:
                     right = self._expect_type(
                         right,
@@ -626,9 +707,12 @@ class Matrix(BaseType):
                     argname="op",
                     extra_message=extra_message,
                 )
-            cfunc_name = f"GrB_Matrix_apply_BinaryOp2nd_{right.dtype}"
-            args = [self, _CScalar(right)]
-            expr_repr = "{0.name}.apply({op}, right={1})"
+            if right._is_cscalar:
+                cfunc_name = f"GrB_Matrix_apply_BinaryOp2nd_{right.dtype}"
+            else:
+                cfunc_name = "GrB_Matrix_apply_BinaryOp2nd_Scalar"
+            args = [self, right]
+            expr_repr = "{0.name}.apply({op}, right={1._expr_name})"
         else:
             raise TypeError("Cannot provide both `left` and `right` to apply")
         return MatrixExpression(
@@ -715,11 +799,14 @@ class Matrix(BaseType):
         )
         return self.reduce_columnwise(op)
 
-    def reduce_scalar(self, op=monoid.plus):
+    def reduce_scalar(self, op=monoid.plus, *, allow_empty=True):
         """
         GrB_Matrix_reduce
         Reduce all values into a scalar
         Default op is monoid.lor for boolean and monoid.plus otherwise
+
+        For empty Matrix objects, the result will be the monoid identity if
+        `allow_empty` is False or empty Scalar if `allow_empty` is True.
         """
         method_name = "reduce_scalar"
         op = get_typed_op(op, self.dtype, kind="binary")
@@ -727,24 +814,28 @@ class Matrix(BaseType):
             op = op.monoid
         else:
             self._expect_op(op, ("Monoid", "Aggregator"), within=method_name, argname="op")
-        if op.opclass == "Aggregator" and op.name in {
-            "argmin",
-            "argmax",
-            "first_index",
-            "last_index",
-        }:
-            raise ValueError(f"Aggregator {op.name} may not be used with Matrix.reduce_scalar.")
+        if op.opclass == "Aggregator":
+            if op.name in {"argmin", "argmax", "first_index", "last_index"}:
+                raise ValueError(f"Aggregator {op.name} may not be used with Matrix.reduce_scalar.")
+            if not allow_empty and op.parent._monoid is None:
+                # But we still kindly allow it if it's a monoid-only aggregator such as sum
+                raise ValueError("allow_empty=False not allowed when using Aggregators")
+        if allow_empty:
+            cfunc_name = "GrB_Matrix_reduce_Monoid_Scalar"
+        else:
+            cfunc_name = "GrB_Matrix_reduce_{output_dtype}"
         return ScalarExpression(
             method_name,
-            "GrB_Matrix_reduce_{output_dtype}",
+            cfunc_name,
             [self],
             op=op,  # to be determined later
+            is_cscalar=not allow_empty,
         )
 
     ##################################
     # Extract and Assign index methods
     ##################################
-    def _extract_element(self, resolved_indexes, dtype=None, name="s_extract"):
+    def _extract_element(self, resolved_indexes, dtype=None, *, is_cscalar, name=None, result=None):
         if dtype is None:
             dtype = self.dtype
         else:
@@ -752,39 +843,44 @@ class Matrix(BaseType):
         rowidx, colidx = resolved_indexes.indices
         if self._is_transposed:
             rowidx, colidx = colidx, rowidx
-        result = Scalar.new(dtype, name=name)
-        if (
-            call(
-                f"GrB_Matrix_extractElement_{dtype}",
-                [_Pointer(result), self, rowidx.index, colidx.index],
-            )
-            is not NoValue
-        ):
-            result._is_empty = False
+        if result is None:
+            result = Scalar.new(dtype, is_cscalar=is_cscalar, name=name)
+        if is_cscalar:
+            if (
+                call(
+                    f"GrB_Matrix_extractElement_{dtype}",
+                    [_Pointer(result), self, rowidx.index, colidx.index],
+                )
+                is not NoValue
+            ):
+                result._empty = False
+        else:
+            call("GrB_Matrix_extractElement_Scalar", [result, self, rowidx.index, colidx.index])
         return result
 
     def _prep_for_extract(self, resolved_indexes):
         method_name = "__getitem__"
-        (rowsize, rows, rowscalar), (colsize, cols, colscalar) = resolved_indexes.indices
-        if rowsize is None:
+        rowidx, colidx = resolved_indexes.indices
+
+        if rowidx.size is None:
             # Row-only selection; GraphBLAS doesn't have this method, so we hack it using transpose
             return VectorExpression(
                 method_name,
                 "GrB_Col_extract",
-                [self, cols, colscalar, rows],
-                expr_repr="{0.name}[{3}, [{2} cols]]",
-                size=colsize,
+                [self, colidx, colidx.cscalar, rowidx],
+                expr_repr="{0.name}[{3._expr_name}, {1._expr_name}]",
+                size=colidx.size,
                 dtype=self.dtype,
                 at=not self._is_transposed,
             )
-        elif colsize is None:
+        elif colidx.size is None:
             # Column-only selection
             return VectorExpression(
                 method_name,
                 "GrB_Col_extract",
-                [self, rows, rowscalar, cols],
-                expr_repr="{0.name}[[{2} rows], {3}]",
-                size=rowsize,
+                [self, rowidx, rowidx.cscalar, colidx],
+                expr_repr="{0.name}[{1._expr_name}, {3._expr_name}]",
+                size=rowidx.size,
                 dtype=self.dtype,
                 at=self._is_transposed,
             )
@@ -792,10 +888,10 @@ class Matrix(BaseType):
             return MatrixExpression(
                 method_name,
                 "GrB_Matrix_extract",
-                [self, rows, rowscalar, cols, colscalar],
-                expr_repr="{0.name}[[{2} rows], [{4} cols]]",
-                nrows=rowsize,
-                ncols=colsize,
+                [self, rowidx, rowidx.cscalar, colidx, colidx.cscalar],
+                expr_repr="{0.name}[{1._expr_name}, {3._expr_name}]",
+                nrows=rowidx.size,
+                ncols=colidx.size,
                 dtype=self.dtype,
                 at=self._is_transposed,
             )
@@ -804,7 +900,7 @@ class Matrix(BaseType):
         rowidx, colidx = resolved_indexes.indices
         if type(value) is not Scalar:
             try:
-                value = Scalar.from_value(value, name="")
+                value = Scalar.from_value(value, is_cscalar=None, name="")
             except TypeError:
                 value = self._expect_type(
                     value,
@@ -813,15 +909,25 @@ class Matrix(BaseType):
                     argname="value",
                     extra_message="Literal scalars also accepted.",
                 )
-        # should we cast?
-        call(
-            f"GrB_Matrix_setElement_{value.dtype}",
-            [self, _CScalar(value), rowidx.index, colidx.index],
-        )
+        if value._is_cscalar:
+            if value._empty:
+                call("GrB_Matrix_removeElement", [self, rowidx.index, colidx.index])
+                return
+            cfunc_name = f"GrB_Matrix_setElement_{value.dtype}"
+        else:
+            cfunc_name = "GrB_Matrix_setElement_Scalar"
+        call(cfunc_name, [self, value, rowidx.index, colidx.index])
 
     def _prep_for_assign(self, resolved_indexes, value, mask=None, is_submask=False):
         method_name = "__setitem__"
-        (rowsize, rows, rowscalar), (colsize, cols, colscalar) = resolved_indexes.indices
+        rowidx, colidx = resolved_indexes.indices
+        rowsize = rowidx.size
+        rows = rowidx.index
+        rowscalar = rowidx.cscalar
+        colsize = colidx.size
+        cols = colidx.index
+        colscalar = colidx.cscalar
+
         extra_message = "Literal scalars also accepted."
 
         value_type = output_type(value)
@@ -844,13 +950,13 @@ class Matrix(BaseType):
                     else:
                         # C(M)[i, J] << v
                         # Upcast v to a Matrix and use Matrix_assign
-                        rows = _CArray([rows.scalar.value])
-                        rowscalar = _CScalar(1)
+                        rows = _CArray([rows.value])
+                        rowscalar = _as_scalar(1, _INDEX, is_cscalar=True)
                         expr = MatrixExpression(
                             method_name,
                             "GrB_Matrix_assign",
                             [value._as_matrix(), rows, rowscalar, cols, colscalar],
-                            expr_repr="[[{2} rows], [{4} cols]] = {0.name}",
+                            expr_repr="[[{2._expr_name} rows], [{4._expr_name} cols]] = {0.name}",
                             nrows=self._nrows,
                             ncols=self._ncols,
                             dtype=self.dtype,
@@ -861,12 +967,14 @@ class Matrix(BaseType):
                         # C[i, J](m) << v
                         # SS, SuiteSparse-specific: subassign
                         cfunc_name = "GrB_Row_subassign"
-                        expr_repr = "[{1}, [{3} cols]](%s) << {0.name}" % mask.name
+                        expr_repr = (
+                            "[{1._expr_name}, [{3._expr_name} cols]](%s) << {0.name}" % mask.name
+                        )
                     else:
                         # C(m)[i, J] << v
                         # C[i, J] << v
                         cfunc_name = "GrB_Row_assign"
-                        expr_repr = "[{1}, [{3} cols]] = {0.name}"
+                        expr_repr = "[{1._expr_name}, [{3._expr_name} cols]] = {0.name}"
                     expr = MatrixExpression(
                         method_name,
                         cfunc_name,
@@ -888,13 +996,13 @@ class Matrix(BaseType):
                     else:
                         # C(M)[I, j] << v
                         # Upcast v to a Matrix and use Matrix_assign
-                        cols = _CArray([cols.scalar.value])
-                        colscalar = _CScalar(1)
+                        cols = _CArray([cols.value])
+                        colscalar = _as_scalar(1, _INDEX, is_cscalar=True)
                         expr = MatrixExpression(
                             method_name,
                             "GrB_Matrix_assign",
                             [value._as_matrix(), rows, rowscalar, cols, colscalar],
-                            expr_repr="[[{2} rows], [{4} cols]] = {0.name}",
+                            expr_repr="[[{2._expr_name} rows], [{4._expr_name} cols]] = {0.name}",
                             nrows=self._nrows,
                             ncols=self._ncols,
                             dtype=self.dtype,
@@ -904,12 +1012,14 @@ class Matrix(BaseType):
                         # C[I, j](m) << v
                         # SS, SuiteSparse-specific: subassign
                         cfunc_name = "GrB_Col_subassign"
-                        expr_repr = "[{1}, [{3} cols]](%s) << {0.name}" % mask.name
+                        expr_repr = (
+                            "[{1._expr_name}, [{3._expr_name} cols]](%s) << {0.name}" % mask.name
+                        )
                     else:
                         # C(m)[I, j] << v
                         # C[I, j] << v
                         cfunc_name = "GrB_Col_assign"
-                        expr_repr = "[{1}, [{3} cols]] = {0.name}"
+                        expr_repr = "[{1._expr_name}, [{3._expr_name} cols]] = {0.name}"
                     expr = MatrixExpression(
                         method_name,
                         cfunc_name,
@@ -964,12 +1074,14 @@ class Matrix(BaseType):
                 # C[I, J](M) << A
                 # SS, SuiteSparse-specific: subassign
                 cfunc_name = "GrB_Matrix_subassign"
-                expr_repr = "[[{2} rows], [{4} cols]](%s) << {0.name}" % mask.name
+                expr_repr = (
+                    "[[{2._expr_name} rows], [{4._expr_name} cols]](%s) << {0.name}" % mask.name
+                )
             else:
                 # C[I, J] << A
                 # C(M)[I, J] << A
                 cfunc_name = "GrB_Matrix_assign"
-                expr_repr = "[[{2} rows], [{4} cols]] = {0.name}"
+                expr_repr = "[[{2._expr_name} rows], [{4._expr_name} cols]] = {0.name}"
             expr = MatrixExpression(
                 method_name,
                 cfunc_name,
@@ -983,7 +1095,7 @@ class Matrix(BaseType):
         else:
             if type(value) is not Scalar:
                 try:
-                    value = Scalar.from_value(value, name="")
+                    value = Scalar.from_value(value, is_cscalar=None, name="")
                 except TypeError:
                     if rowsize is None or colsize is None:
                         types = (Scalar, Vector)
@@ -1003,13 +1115,15 @@ class Matrix(BaseType):
                         # SS, SuiteSparse-specific: subassign
                         cfunc_name = "GrB_Row_subassign"
                         value_vector = Vector.new(value.dtype, size=mask.mask._size, name="v_temp")
-                        expr_repr = "[{1}, [{3} cols]](%s) << {0.name}" % mask.name
+                        expr_repr = (
+                            "[{1._expr_name}, [{3._expr_name} cols]](%s) << {0.name}" % mask.name
+                        )
                     else:
                         # C(m)[i, J] << c
                         # C[i, J] << c
                         cfunc_name = "GrB_Row_assign"
                         value_vector = Vector.new(value.dtype, size=colsize, name="v_temp")
-                        expr_repr = "[{1}, [{3} cols]] = {0.name}"
+                        expr_repr = "[{1._expr_name}, [{3._expr_name} cols]] = {0.name}"
                     # SS, SuiteSparse-specific: assume efficient vector with single scalar
                     value_vector << value
 
@@ -1042,7 +1156,7 @@ class Matrix(BaseType):
                         method_name,
                         cfunc_name,
                         [value_vector, rows, rowscalar, cols],
-                        expr_repr="[[{2} rows], {3}] = {0.name}",
+                        expr_repr="[[{2._expr_name} rows], {3._expr_name}] = {0.name}",
                         nrows=self._nrows,
                         ncols=self._ncols,
                         dtype=self.dtype,
@@ -1074,25 +1188,34 @@ class Matrix(BaseType):
                             )
                     # C[I, J](M) << c
                     # SS, SuiteSparse-specific: subassign
-                    cfunc_name = f"GrB_Matrix_subassign_{value.dtype}"
-                    expr_repr = "[[{2} rows], [{4} cols]](%s) = {0}" % mask.name
+                    if value._is_cscalar:
+                        cfunc_name = f"GrB_Matrix_subassign_{value.dtype}"
+                    else:
+                        cfunc_name = "GrB_Matrix_subassign_Scalar"
+                    expr_repr = (
+                        "[[{2._expr_name} rows], [{4._expr_name} cols]](%s) = {0._expr_name}"
+                        % mask.name
+                    )
                 else:
                     # C(M)[I, J] << c
                     # C(M)[i, J] << c
                     # C(M)[I, j] << c
                     # C(M)[i, j] << c
                     if rowsize is None:
-                        rows = _CArray([rows.scalar.value])
-                        rowscalar = _CScalar(1)
+                        rows = _CArray([rows.value])
+                        rowscalar = _as_scalar(1, _INDEX, is_cscalar=True)
                     if colsize is None:
-                        cols = _CArray([cols.scalar.value])
-                        colscalar = _CScalar(1)
-                    cfunc_name = f"GrB_Matrix_assign_{value.dtype}"
-                    expr_repr = "[[{2} rows], [{4} cols]] = {0}"
+                        cols = _CArray([cols.value])
+                        colscalar = _as_scalar(1, _INDEX, is_cscalar=True)
+                    if value._is_cscalar:
+                        cfunc_name = f"GrB_Matrix_assign_{value.dtype}"
+                    else:
+                        cfunc_name = "GrB_Matrix_assign_Scalar"
+                    expr_repr = "[[{2._expr_name} rows], [{4._expr_name} cols]] = {0._expr_name}"
                 expr = MatrixExpression(
                     method_name,
                     cfunc_name,
-                    [_CScalar(value), rows, rowscalar, cols, colscalar],
+                    [value, rows, rowscalar, cols, colscalar],
                     expr_repr=expr_repr,
                     nrows=self._nrows,
                     ncols=self._ncols,
@@ -1214,7 +1337,7 @@ class MatrixExpression(BaseExpression):
     def shape(self):
         return (self._nrows, self._ncols)
 
-    # Paste here from _automethods.py
+    # Begin auto-generated code: Matrix
     _get_value = _automethods._get_value
     S = wrapdoc(Matrix.S)(property(_automethods.S))
     T = wrapdoc(Matrix.T)(property(_automethods.T))
@@ -1234,6 +1357,7 @@ class MatrixExpression(BaseExpression):
     apply = wrapdoc(Matrix.apply)(property(_automethods.apply))
     ewise_add = wrapdoc(Matrix.ewise_add)(property(_automethods.ewise_add))
     ewise_mult = wrapdoc(Matrix.ewise_mult)(property(_automethods.ewise_mult))
+    ewise_union = wrapdoc(Matrix.ewise_union)(property(_automethods.ewise_union))
     gb_obj = wrapdoc(Matrix.gb_obj)(property(_automethods.gb_obj))
     isclose = wrapdoc(Matrix.isclose)(property(_automethods.isclose))
     isequal = wrapdoc(Matrix.isequal)(property(_automethods.isequal))
@@ -1253,8 +1377,8 @@ class MatrixExpression(BaseExpression):
     to_values = wrapdoc(Matrix.to_values)(property(_automethods.to_values))
     wait = wrapdoc(Matrix.wait)(property(_automethods.wait))
     # These raise exceptions
-    __array__ = wrapdoc(Matrix.__array__)(Matrix.__array__)
-    __bool__ = wrapdoc(Matrix.__bool__)(Matrix.__bool__)
+    __array__ = Matrix.__array__
+    __bool__ = Matrix.__bool__
     __iadd__ = _automethods.__iadd__
     __iand__ = _automethods.__iand__
     __ifloordiv__ = _automethods.__ifloordiv__
@@ -1266,6 +1390,86 @@ class MatrixExpression(BaseExpression):
     __isub__ = _automethods.__isub__
     __itruediv__ = _automethods.__itruediv__
     __ixor__ = _automethods.__ixor__
+    # End auto-generated code: Matrix
+
+
+class MatrixIndexExpr(AmbiguousAssignOrExtract):
+    __slots__ = "_ncols", "_nrows"
+    ndim = 2
+    output_type = Matrix
+    _is_transposed = False
+
+    def __init__(self, parent, resolved_indexes, nrows, ncols):
+        super().__init__(parent, resolved_indexes)
+        self._nrows = nrows
+        self._ncols = ncols
+
+    @property
+    def ncols(self):
+        return self._ncols
+
+    @property
+    def nrows(self):
+        return self._nrows
+
+    @property
+    def shape(self):
+        return (self._nrows, self._ncols)
+
+    # Begin auto-generated code: Matrix
+    _get_value = _automethods._get_value
+    S = wrapdoc(Matrix.S)(property(_automethods.S))
+    T = wrapdoc(Matrix.T)(property(_automethods.T))
+    V = wrapdoc(Matrix.V)(property(_automethods.V))
+    __and__ = wrapdoc(Matrix.__and__)(property(_automethods.__and__))
+    __contains__ = wrapdoc(Matrix.__contains__)(property(_automethods.__contains__))
+    __getitem__ = wrapdoc(Matrix.__getitem__)(property(_automethods.__getitem__))
+    __iter__ = wrapdoc(Matrix.__iter__)(property(_automethods.__iter__))
+    __matmul__ = wrapdoc(Matrix.__matmul__)(property(_automethods.__matmul__))
+    __or__ = wrapdoc(Matrix.__or__)(property(_automethods.__or__))
+    __rand__ = wrapdoc(Matrix.__rand__)(property(_automethods.__rand__))
+    __rmatmul__ = wrapdoc(Matrix.__rmatmul__)(property(_automethods.__rmatmul__))
+    __ror__ = wrapdoc(Matrix.__ror__)(property(_automethods.__ror__))
+    _carg = wrapdoc(Matrix._carg)(property(_automethods._carg))
+    _name_html = wrapdoc(Matrix._name_html)(property(_automethods._name_html))
+    _nvals = wrapdoc(Matrix._nvals)(property(_automethods._nvals))
+    apply = wrapdoc(Matrix.apply)(property(_automethods.apply))
+    ewise_add = wrapdoc(Matrix.ewise_add)(property(_automethods.ewise_add))
+    ewise_mult = wrapdoc(Matrix.ewise_mult)(property(_automethods.ewise_mult))
+    ewise_union = wrapdoc(Matrix.ewise_union)(property(_automethods.ewise_union))
+    gb_obj = wrapdoc(Matrix.gb_obj)(property(_automethods.gb_obj))
+    isclose = wrapdoc(Matrix.isclose)(property(_automethods.isclose))
+    isequal = wrapdoc(Matrix.isequal)(property(_automethods.isequal))
+    kronecker = wrapdoc(Matrix.kronecker)(property(_automethods.kronecker))
+    mxm = wrapdoc(Matrix.mxm)(property(_automethods.mxm))
+    mxv = wrapdoc(Matrix.mxv)(property(_automethods.mxv))
+    name = wrapdoc(Matrix.name)(property(_automethods.name))
+    name = name.setter(_automethods._set_name)
+    nvals = wrapdoc(Matrix.nvals)(property(_automethods.nvals))
+    reduce_columns = wrapdoc(Matrix.reduce_columns)(property(_automethods.reduce_columns))
+    reduce_columnwise = wrapdoc(Matrix.reduce_columnwise)(property(_automethods.reduce_columnwise))
+    reduce_rows = wrapdoc(Matrix.reduce_rows)(property(_automethods.reduce_rows))
+    reduce_rowwise = wrapdoc(Matrix.reduce_rowwise)(property(_automethods.reduce_rowwise))
+    reduce_scalar = wrapdoc(Matrix.reduce_scalar)(property(_automethods.reduce_scalar))
+    ss = wrapdoc(Matrix.ss)(property(_automethods.ss))
+    to_pygraphblas = wrapdoc(Matrix.to_pygraphblas)(property(_automethods.to_pygraphblas))
+    to_values = wrapdoc(Matrix.to_values)(property(_automethods.to_values))
+    wait = wrapdoc(Matrix.wait)(property(_automethods.wait))
+    # These raise exceptions
+    __array__ = Matrix.__array__
+    __bool__ = Matrix.__bool__
+    __iadd__ = _automethods.__iadd__
+    __iand__ = _automethods.__iand__
+    __ifloordiv__ = _automethods.__ifloordiv__
+    __imatmul__ = _automethods.__imatmul__
+    __imod__ = _automethods.__imod__
+    __imul__ = _automethods.__imul__
+    __ior__ = _automethods.__ior__
+    __ipow__ = _automethods.__ipow__
+    __isub__ = _automethods.__isub__
+    __itruediv__ = _automethods.__itruediv__
+    __ixor__ = _automethods.__ixor__
+    # End auto-generated code: Matrix
 
 
 class TransposedMatrix:
@@ -1340,6 +1544,7 @@ class TransposedMatrix:
     # Delayed methods
     ewise_add = Matrix.ewise_add
     ewise_mult = Matrix.ewise_mult
+    ewise_union = Matrix.ewise_union
     mxv = Matrix.mxv
     mxm = Matrix.mxm
     kronecker = Matrix.kronecker
@@ -1374,6 +1579,7 @@ class TransposedMatrix:
     # Misc.
     isequal = Matrix.isequal
     isclose = Matrix.isclose
+    wait = Matrix.wait
     _extract_element = Matrix._extract_element
     _prep_for_extract = Matrix._prep_for_extract
     __eq__ = Matrix.__eq__
@@ -1387,6 +1593,7 @@ class TransposedMatrix:
 
 
 utils._output_types[Matrix] = Matrix
+utils._output_types[MatrixIndexExpr] = Matrix
 utils._output_types[MatrixExpression] = Matrix
 utils._output_types[TransposedMatrix] = TransposedMatrix
 
