@@ -276,7 +276,7 @@ def test_monoid_parameterized():
     # identity may be a value
     def logaddexp(base):
         def inner(x, y):
-            return np.log(base ** x + base ** y) / np.log(base)
+            return np.log(base**x + base**y) / np.log(base)
 
         return inner
 
@@ -778,15 +778,15 @@ def test_div_semirings():
     A1 = Matrix.from_values([0, 1], [0, 0], [-1, -3])
     A2 = Matrix.from_values([0, 1], [0, 0], [2, 2])
     result = A1.T.mxm(A2, semiring.plus_cdiv).new()
-    assert result[0, 0].value == -1
+    assert result[0, 0].new() == -1
     assert result.dtype == dtypes.INT64
 
     result = A1.T.mxm(A2, semiring.plus_truediv).new()
-    assert result[0, 0].value == -2
+    assert result[0, 0].new() == -2
     assert result.dtype == dtypes.FP64
 
     result = A1.T.mxm(A2, semiring.plus_floordiv).new()
-    assert result[0, 0].value == -3
+    assert result[0, 0].new() == -3
     assert result.dtype == dtypes.INT64
 
 
@@ -978,3 +978,25 @@ def test_lazy_op():
         unary.from_string("misc.lazy.badpath")
     assert op.from_string("lazy") is unary.lazy
     assert op.from_string("numpy.lazy") is unary.numpy.lazy
+
+
+def test_positional():
+    assert unary.positioni.is_positional
+    assert unary.positioni1[int].is_positional
+    assert unary.positionj1.is_positional
+    assert unary.positionj[float].is_positional
+    assert not unary.exp.is_positional
+    assert not unary.abs[bool].is_positional
+
+    assert binary.firsti.is_positional
+    assert binary.secondj1[int].is_positional
+    assert not binary.plus.is_positional
+    assert not binary.minus[float].is_positional
+
+    assert not monoid.plus.is_positional
+    assert not monoid.plus[int].is_positional
+
+    assert semiring.any_firsti.is_positional
+    assert semiring.any_secondj[int].is_positional
+    assert not semiring.any_first.is_positional
+    assert not semiring.any_second[int].is_positional
