@@ -1,6 +1,7 @@
 import inspect
 import itertools
 import pickle
+import sys
 import weakref
 
 import numpy as np
@@ -11,6 +12,7 @@ import grblas
 from grblas import agg, binary, dtypes, monoid, semiring, unary
 from grblas.exceptions import (
     DimensionMismatch,
+    EmptyObject,
     IndexOutOfBound,
     InvalidValue,
     OutputNotEmpty,
@@ -45,13 +47,13 @@ def test_new():
 
 
 def test_large_vector():
-    u = Vector.from_values([0, 2 ** 59], [0, 1])
-    assert u.size == 2 ** 59 + 1
-    assert u[2 ** 59].value == 1
+    u = Vector.from_values([0, 2**59], [0, 1])
+    assert u.size == 2**59 + 1
+    assert u[2**59].new() == 1
     with pytest.raises(InvalidValue):
-        Vector.from_values([0, 2 ** 64 - 2], [0, 1])
+        Vector.from_values([0, 2**64 - 2], [0, 1])
     with pytest.raises(OverflowError):
-        Vector.from_values([0, 2 ** 64], [0, 1])
+        Vector.from_values([0, 2**64], [0, 1])
 
 
 def test_dup(v):
@@ -62,7 +64,7 @@ def test_dup(v):
     assert u.size == v.size
     # Ensure they are not the same backend object
     v[0] = 1000
-    assert u[0].value != 1000
+    assert u[0].new() != 1000
     # extended functionality
     w = Vector.from_values([0, 1], [0, 2.5], dtype=dtypes.FP64)
     x = w.dup(dtype=dtypes.INT64)
@@ -86,7 +88,7 @@ def test_from_values():
     assert u3.size == 10
     assert u3.nvals == 2  # duplicates were combined
     assert u3.dtype == int
-    assert u3[1].value == 6  # 2*3
+    assert u3[1].new() == 6  # 2*3
     with pytest.raises(ValueError, match="Duplicate indices found"):
         # Duplicate indices requires a dup_op
         Vector.from_values([0, 1, 1], [True, True, True])
@@ -150,7 +152,7 @@ def test_resize(v):
     v.resize(20)
     assert v.size == 20
     assert v.nvals == 4
-    assert compute(v[19].value) is None
+    assert compute(v[19].new().value) is None
     v.resize(4)
     assert v.size == 4
     assert v.nvals == 2
@@ -229,30 +231,35 @@ def test_extract_input_mask():
 
 
 def test_extract_element(v):
-    assert v[1].value == 1
+    assert v[1].new() == 1
+    with pytest.raises(TypeError, match="enable automatic"):
+        v[1].value
     assert v[6].new() == 0
     with pytest.raises(TypeError, match="Invalid type for index"):
         v[object()]
     with pytest.raises(IndexError):
         v[100]
+    s = Scalar.new(int)
+    s << v[1]
+    assert s == 1
 
 
 def test_set_element(v):
-    assert compute(v[0].value) is None
-    assert v[1].value == 1
+    assert compute(v[0].new().value) is None
+    assert v[1].new() == 1
     v[0] = 12
     v[1] << 9
-    assert v[0].value == 12
+    assert v[0].new() == 12
     assert v[1].new() == 9
 
 
 def test_remove_element(v):
-    assert v[1].value == 1
+    assert v[1].new() == 1
     del v[1]
-    assert compute(v[1].value) is None
-    assert v[4].value == 2
-    with pytest.raises(TypeError, match="Remove Element only supports"):
-        del v[1:3]
+    assert compute(v[1].new().value) is None
+    assert v[4].new() == 2
+    # with pytest.raises(TypeError, match="Remove Element only supports"):
+    del v[1:3]  # Now okay
 
 
 def test_vxm(v, A):
@@ -318,6 +325,13 @@ def test_vxm_accum(v, A):
     w3 = v.dup()
     w3(accum=monoid.plus) << v.vxm(A, semiring.plus_times)
     assert w3.isequal(result)
+    # allow strings for accum
+    w4 = v.dup()
+    w4("+") << v.vxm(A, semiring.plus_times)
+    assert w4.isequal(result)
+    w5 = v.dup()
+    w5(accum="plus") << v.vxm(A, semiring.plus_times)
+    assert w5.isequal(result)
 
 
 def test_ewise_mult(v):
@@ -434,6 +448,21 @@ def test_extract_fancy_scalars(v):
         t(accum=binary.plus) << v[0]
 
 
+def test_extract_negative_indices(v):
+    assert v[-1].new() == 0
+    assert compute(v[-v.size].new().value) is None
+    assert v[[-v.size]].new().nvals == 0
+    assert v[Scalar.from_value(-4)].new() == 1
+    w = v[[-1, -3]].new()
+    assert w.isequal(Vector.from_values([0, 1], [0, 2]))
+    with pytest.raises(IndexError):
+        v[-v.size - 1]
+    with pytest.raises(IndexError):
+        v[Scalar.from_value(-v.size - 1)]
+    with pytest.raises(IndexError):
+        v[[-v.size - 1]]
+
+
 def test_assign(v):
     u = Vector.from_values([0, 2], [9, 8])
     result = Vector.from_values([0, 1, 3, 4, 6], [9, 1, 1, 8, 0])
@@ -469,6 +498,8 @@ def test_assign_scalar(v):
         w[1] = object()
     w << 2
     assert w.isequal(Vector.from_values([0, 1, 2], [2, 2, 2]))
+    w[0] = Scalar.new(int)
+    assert w.isequal(Vector.from_values([1, 2], [2, 2]))
 
 
 def test_assign_scalar_mask(v):
@@ -605,6 +636,12 @@ def test_apply_binary(v):
     assert w1.isequal(w3)
 
 
+def test_apply_empty(v):
+    s = Scalar.new(int, is_cscalar=True)
+    with pytest.raises(EmptyObject):
+        v.apply(binary.plus, s).new()
+
+
 def test_reduce(v):
     s = v.reduce(monoid.plus).new()
     assert s == 4
@@ -623,6 +660,26 @@ def test_reduce(v):
     with pytest.raises(KeyError, match="plus does not work"):
         # KeyError here is kind of weird
         b1.reduce()
+    # v is not empty, so it shouldn't matter how we reduce
+    for allow_empty, is_cscalar in itertools.product([True, False], [True, False]):
+        t = v.reduce(allow_empty=allow_empty).new(is_cscalar=is_cscalar)
+        assert t == 4
+
+
+def test_reduce_empty():
+    w = Vector.new(int, 5)
+    s = Scalar.from_value(16)
+    s(accum=binary.times) << w.reduce(monoid.plus, allow_empty=True)
+    assert s == 16
+    s(accum=binary.times) << w.reduce(monoid.plus, allow_empty=False)
+    assert s == 0
+    assert w.reduce(monoid.plus, allow_empty=True).new().is_empty
+    assert w.reduce(monoid.plus, allow_empty=False).new() == 0
+    assert w.reduce(agg.sum, allow_empty=True).new().is_empty
+    assert w.reduce(agg.sum, allow_empty=False).new() == 0
+    assert w.reduce(agg.mean, allow_empty=True).new().is_empty
+    with pytest.raises(ValueError):
+        w.reduce(agg.mean, allow_empty=False)
 
 
 def test_reduce_agg(v):
@@ -637,22 +694,22 @@ def test_reduce_agg(v):
     assert v.reduce(agg.count_nonzero).new() == 3
     assert v.reduce(agg.count_zero).new() == 1
     assert v.reduce(agg.sum_of_squares).new() == 6
-    assert v.reduce(agg.hypot).new().isclose(6 ** 0.5)
-    assert v.reduce(agg.logaddexp).new().isclose(np.log(1 + 2 * np.e + np.e ** 2))
+    assert v.reduce(agg.hypot).new().isclose(6**0.5)
+    assert v.reduce(agg.logaddexp).new().isclose(np.log(1 + 2 * np.e + np.e**2))
     assert v.reduce(agg.logaddexp2).new().isclose(np.log2(9))
     assert v.reduce(agg.mean).new() == 1
     assert v.reduce(agg.peak_to_peak).new() == 2
     assert v.reduce(agg.varp).new().isclose(0.5)
     assert v.reduce(agg.vars).new().isclose(2 / 3)
-    assert v.reduce(agg.stdp).new().isclose(0.5 ** 0.5)
+    assert v.reduce(agg.stdp).new().isclose(0.5**0.5)
     assert v.reduce(agg.stds).new().isclose((2 / 3) ** 0.5)
     assert v.reduce(agg.L0norm).new() == 3
     assert v.reduce(agg.L1norm).new() == 4
-    assert v.reduce(agg.L2norm).new().isclose(6 ** 0.5)
+    assert v.reduce(agg.L2norm).new().isclose(6**0.5)
     assert v.reduce(agg.Linfnorm).new() == 2
     assert v.reduce(agg.exists).new() == 1
     w = binary.plus(v, 1).new()
-    assert w.reduce(agg.geometric_mean).new().isclose(12 ** 0.25)
+    assert w.reduce(agg.geometric_mean).new().isclose(12**0.25)
     assert w.reduce(agg.harmonic_mean).new().isclose(12 / 7)
 
     silly = agg.Aggregator(
@@ -662,7 +719,7 @@ def test_reduce_agg(v):
         types=[agg.varp],
     )
     s = v.reduce(silly).new()
-    assert s.isclose(0.5 ** 1.5)
+    assert s.isclose(0.5**1.5)
 
     s = Vector.new(int, size=5).reduce(silly).new()
     assert s.is_empty
@@ -1116,13 +1173,13 @@ def test_not_to_array(v):
 def test_vector_index_with_scalar():
     v = Vector.from_values([0, 1, 2], [10, 20, 30])
     expected = Vector.from_values([0, 1], [20, 10])
-    for dtype in ["bool", "int8", "uint8", "int16", "uint16", "int32", "uint32"]:
+    for dtype in ["int8", "uint8", "int16", "uint16", "int32", "uint32"]:
         s1 = Scalar.from_value(1, dtype=dtype)
-        assert v[s1] == 20
+        assert v[s1].new() == 20
         s0 = Scalar.from_value(0, dtype=dtype)
         w = v[[s1, s0]].new()
         assert w.isequal(expected)
-    for dtype in ["fp32", "fp64", "fc32", "fc64"]:
+    for dtype in ["bool", "fp32", "fp64", "fc32", "fc64"]:
         s = Scalar.from_value(1, dtype=dtype)
         with pytest.raises(TypeError, match="An integer is required for indexing"):
             v[s]
@@ -1157,6 +1214,20 @@ def test_inner(v):
     expected = R.mxm(C).new()[0, 0].new()
     assert expected.isequal(v.inner(v).new())
     assert expected.isequal((v @ v).new())
+    s = Scalar.new(v.dtype)
+    s << v.inner(v)
+    assert s == 6
+    s(binary.plus) << v.inner(v)
+    assert s == 12
+
+
+@autocompute
+def test_inner_infix(v):
+    s = Scalar.new(v.dtype)
+    s << v @ v
+    assert s == 6
+    s(binary.plus) << v @ v
+    assert s == 12
 
 
 def test_outer(v):
@@ -1253,9 +1324,10 @@ def test_auto_assign(v):
     expected[:3] = expr.new()
     v[:3] = expr
     assert expected.isequal(v)
-    with pytest.raises(TypeError):
-        # Not yet supported, but we could!
-        v[:3] = v[1:4]
+    v[:3] = v[1:4]
+    del expected[0]
+    expected[1] = 1
+    assert expected.isequal(v)
 
 
 @autocompute
@@ -1271,12 +1343,12 @@ def test_expr_is_like_vector(v):
         "__delitem__",
         "__lshift__",
         "__setitem__",
-        "_as_matrix",
         "_assign_element",
         "_delete_element",
         "_deserialize",
         "_extract_element",
         "_name_counter",
+        "_parent",
         "_prep_for_assign",
         "_prep_for_extract",
         "_update",
@@ -1288,10 +1360,34 @@ def test_expr_is_like_vector(v):
         "update",
     }
     assert attrs - expr_attrs == expected
-    assert attrs - infix_attrs == expected | {
-        "_expect_op",
-        "_expect_type",
+    assert attrs - infix_attrs == expected
+
+
+@autocompute
+def test_index_expr_is_like_vector(v):
+    w = v.dup(dtype=bool)
+    attrs = {attr for attr, val in inspect.getmembers(w)}
+    expr_attrs = {attr for attr, val in inspect.getmembers(w[[0, 1]])}
+    expected = {
+        "__del__",
+        "__delitem__",
+        "__setitem__",
+        "_assign_element",
+        "_delete_element",
+        "_deserialize",
+        "_extract_element",
+        "_name_counter",
+        "_parent",
+        "_prep_for_assign",
+        "_prep_for_extract",
+        "_update",
+        "build",
+        "clear",
+        "from_pygraphblas",
+        "from_values",
+        "resize",
     }
+    assert attrs - expr_attrs == expected
 
 
 def test_random(v):
@@ -1384,6 +1480,99 @@ def test_smallestk(v):
     assert w.isequal(expected)
 
 
+@pytest.mark.parametrize("do_iso", [False, True])
+def test_compactify(do_iso):
+    orig_indices = [1, 3, 4, 6]
+    new_indices = [0, 1, 2, 3]
+    if do_iso:
+        v = Vector.from_values(orig_indices, 1)
+    else:
+        v = Vector.from_values(orig_indices, [1, 4, 2, 0])
+
+    def check(v, expected, *args, stop=0, **kwargs):
+        w = v.ss.compactify(*args, **kwargs)
+        assert w.isequal(expected)
+        for n in reversed(range(stop, 5)):
+            expected = expected[:n].new()
+            w = v.ss.compactify(*args, size=n, **kwargs)
+            assert w.isequal(expected)
+
+    def reverse(v):
+        return v[::-1].new().ss.compactify("first", v.size)
+
+    def check_reverse(v, expected, *args, stop=0, **kwargs):
+        w = v.ss.compactify(*args, reverse=True, **kwargs)
+        x = reverse(expected)
+        assert w.isequal(x)
+        for n in reversed(range(stop, 5)):
+            x = reverse(expected[:n].new())
+            w = v.ss.compactify(*args, size=n, reverse=True, **kwargs)
+            assert w.isequal(x)
+
+    expected = Vector.from_values(
+        new_indices,
+        1 if do_iso else [1, 4, 2, 0],
+        size=4,
+    )
+    check(v, expected, "first")
+    check_reverse(v, expected, "first")
+    check(v, reverse(expected), "last")
+    check_reverse(v, reverse(expected), "last")
+
+    expected = Vector.from_values(
+        new_indices,
+        orig_indices,
+        size=4,
+    )
+    check(v, expected, "first", asindex=True)
+    check_reverse(v, expected, "first", asindex=True)
+    check(v, reverse(expected), "last", asindex=True)
+    check_reverse(v, reverse(expected), "last", asindex=True)
+
+    expected = Vector.from_values(
+        new_indices,
+        1 if do_iso else [0, 1, 2, 4],
+        size=4,
+    )
+    check(v, expected, "smallest")
+    check_reverse(v, expected, "smallest")
+    check(v, reverse(expected), "largest")
+    check_reverse(v, reverse(expected), "largest")
+
+    if not do_iso:
+        expected = Vector.from_values(
+            new_indices,
+            [6, 1, 4, 3],
+            size=4,
+        )
+        check(v, expected, "smallest", asindex=True, stop=3)
+        check_reverse(v, expected, "smallest", asindex=True, stop=3)
+        check(v, reverse(expected), "largest", asindex=True, stop=3)
+        check_reverse(v, reverse(expected), "largest", asindex=True, stop=3)
+
+    def compare(v, expected, isequal=True, **kwargs):
+        for _ in range(1000):
+            w = v.ss.compactify("random", **kwargs)
+            if w.isequal(expected) == isequal:
+                break
+        else:
+            raise AssertionError("random failed")
+
+    with pytest.raises(AssertionError):
+        compare(v, v[::-1].new())
+    for asindex in [False, True]:
+        compare(v, v.ss.compactify("first", asindex=asindex), asindex=asindex)
+        compare(v, v.ss.compactify("first", 0, asindex=asindex), size=0, asindex=asindex)
+        for i in range(1, 4):
+            for how in ["first", "last", "smallest", "largest"]:
+                w = v.ss.compactify("first", i, asindex=asindex)
+                compare(v, w, size=i, asindex=asindex)
+                if not do_iso:
+                    compare(v, w, size=i, asindex=asindex, isequal=True)
+    with pytest.raises(ValueError):
+        v.ss.compactify("bad_how")
+
+
 def test_slice():
     v = Vector.from_values(np.arange(5), np.arange(5))
     w = v[0:0].new()
@@ -1403,12 +1592,15 @@ def test_concat(v):
     expected[: v.size] = v
     expected[v.size :] = v
     w1 = grblas.ss.concat([v, v])
-    assert w1.isequal(expected)
+    assert w1.isequal(expected, check_dtype=True)
     w2 = Vector.new(v.dtype, size=2 * v.size)
     w2.ss.concat([v, v])
-    assert w2.isequal(expected)
+    assert w2.isequal(expected, check_dtype=True)
     with pytest.raises(TypeError):
         w2.ss.concat([[v, v]])
+    w3 = grblas.ss.concat([v, v], dtype=float)
+    assert w3.isequal(expected)
+    assert w3.dtype == float
 
 
 def test_split(v):
@@ -1429,3 +1621,43 @@ def test_ndim(A, v):
     assert v.ewise_mult(v).ndim == 1
     assert (v & v).ndim == 1
     assert (A @ v).ndim == 1
+
+
+def test_sizeof(v):
+    assert sys.getsizeof(v) > v.nvals * 16
+
+
+def test_ewise_union():
+    v1 = Vector.from_values([0], [1], size=3)
+    v2 = Vector.from_values([1], [2], size=3)
+    result = v1.ewise_union(v2, binary.plus, 10, 20).new()
+    expected = Vector.from_values([0, 1], [21, 12], size=3)
+    assert result.isequal(expected)
+    # Handle Scalars
+    result = v1.ewise_union(v2, binary.plus, Scalar.from_value(10), Scalar.from_value(20)).new()
+    assert result.isequal(expected)
+    # Upcast if scalars are floats
+    result = v1.ewise_union(v2, monoid.plus, 10.1, 20.2).new()
+    expected = Vector.from_values([0, 1], [21.2, 12.1], size=3)
+    assert result.isclose(expected)
+
+    result = v1.ewise_union(v2, binary.minus, 0, 0).new()
+    expected = Vector.from_values([0, 1], [1, -2], size=3)
+    assert result.isequal(expected)
+    result = (v1 - v2).new()
+    assert result.isequal(expected)
+
+    bad = Vector.new(int, size=1)
+    with pytest.raises(DimensionMismatch):
+        v1.ewise_union(bad, binary.plus, 0, 0)
+    with pytest.raises(TypeError, match="Literal scalars"):
+        v1.ewise_union(v2, binary.plus, v2, 20)
+    with pytest.raises(TypeError, match="Literal scalars"):
+        v1.ewise_union(v2, binary.plus, 10, v2)
+
+
+def test_delete_via_scalar(v):
+    del v[[1, 3]]
+    assert v.isequal(Vector.from_values([4, 6], [2, 0]))
+    del v[:]
+    assert v.nvals == 0
