@@ -10,7 +10,7 @@ from .exceptions import NoValue, check_status
 from .expr import AmbiguousAssignOrExtract, IndexerResolver, Updater
 from .mask import StructuralMask, ValueMask
 from .operator import get_semiring, get_typed_op
-from .scalar import Scalar, ScalarExpression, _CScalar
+from .scalar import _MATERIALIZE, Scalar, ScalarExpression, _as_scalar
 from .utils import (
     _CArray,
     _Pointer,
@@ -30,19 +30,23 @@ class Vector(BaseType):
     High-level wrapper around GrB_Vector type
     """
 
-    __slots__ = "_size", "ss"
+    __slots__ = "_size", "_parent", "ss"
     ndim = 1
     _name_counter = itertools.count()
 
-    def __init__(self, gb_obj, dtype, *, name=None):
+    def __init__(self, gb_obj, dtype, *, parent=None, name=None):
         if name is None:
             name = f"v_{next(Vector._name_counter)}"
         self._size = None
         super().__init__(gb_obj, dtype, name)
         # Add ss extension methods
         self.ss = ss(self)
+        self._parent = parent
 
     def __del__(self):
+        parent = getattr(self, "_parent", None)
+        if parent is not None:
+            return
         gb_obj = getattr(self, "gb_obj", None)
         if gb_obj is not None:
             # it's difficult/dangerous to record the call, b/c `self.name` may not exist
@@ -74,12 +78,21 @@ class Vector(BaseType):
             return format_vector(self, mask=mask)
 
     def _repr_html_(self, mask=None, collapse=False):
+        if self._parent is not None and mask is None:
+            # Scalars repr can't handle mask
+            return self._parent._repr_html_(collapse=collapse)
         from .formatting import format_vector_html
         from .recorder import skip_record
 
         with skip_record:
             return format_vector_html(self, mask=mask, collapse=collapse)
 
+    @property
+    def _name_html(self):
+        if self._parent is not None:
+            return self._parent._name_html
+        return super()._name_html
+
     def __reduce__(self):
         # SS, SuiteSparse-specific: export
         pieces = self.ss.export(raw=True)
@@ -103,25 +116,38 @@ class Vector(BaseType):
 
     def __getitem__(self, keys):
         resolved_indexes = IndexerResolver(self, keys)
-        return AmbiguousAssignOrExtract(self, resolved_indexes)
+        shape = resolved_indexes.shape
+        if not shape:
+            from .scalar import ScalarIndexExpr
+
+            return ScalarIndexExpr(self, resolved_indexes)
+        else:
+            return VectorIndexExpr(self, resolved_indexes, *shape)
 
     def __setitem__(self, keys, expr):
         Updater(self)[keys] = expr
 
     def __contains__(self, index):
         extractor = self[index]
-        if not extractor.resolved_indexes.is_single_element:
+        if not extractor._is_scalar:
             raise TypeError(
                 f"Invalid index to Vector contains: {index!r}.  An integer is expected.  "
                 "Doing `index in my_vector` checks whether a value is present at that index."
             )
         scalar = extractor.new(name="s_contains")
-        return not scalar.is_empty
+        return not scalar._is_empty
 
     def __iter__(self):
+        self.wait()  # sort in SS
         indices, values = self.to_values()
         return indices.flat
 
+    def __sizeof__(self):
+        size = ffi_new("size_t*")
+        scalar = Scalar(size, _INDEX, name="s_size", is_cscalar=True, empty=True)
+        call("GxB_Vector_memoryUsage", [_Pointer(scalar), self])
+        return size[0] + object.__sizeof__(self)
+
     def isequal(self, other, *, check_dtype=False):
         """
         Check for exact equality (same size, same empty values)
@@ -148,7 +174,7 @@ class Vector(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce(monoid.land).new().value
+        return matches.reduce(monoid.land, allow_empty=False).new().value
 
     def isclose(self, other, *, rel_tol=1e-7, abs_tol=0.0, check_dtype=False):
         """
@@ -172,12 +198,12 @@ class Vector(BaseType):
             return False
 
         # Check if all results are True
-        return matches.reduce(monoid.land).new().value
+        return matches.reduce(monoid.land, allow_empty=False).new().value
 
     @property
     def size(self):
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_size", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_size", is_cscalar=True, empty=True)
         call("GrB_Vector_size", [_Pointer(scalar), self])
         return n[0]
 
@@ -188,7 +214,7 @@ class Vector(BaseType):
     @property
     def nvals(self):
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_nvals", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_nvals", is_cscalar=True, empty=True)
         call("GrB_Vector_nvals", [_Pointer(scalar), self])
         return n[0]
 
@@ -203,9 +229,9 @@ class Vector(BaseType):
         call("GrB_Vector_clear", [self])
 
     def resize(self, size):
-        size = _CScalar(size)
+        size = _as_scalar(size, _INDEX, is_cscalar=True)
         call("GrB_Vector_resize", [self, size])
-        self._size = size.scalar.value
+        self._size = size.value
 
     def to_values(self, dtype=None):
         """
@@ -216,7 +242,7 @@ class Vector(BaseType):
         indices = _CArray(size=nvals, name="&index_array")
         values = _CArray(size=nvals, dtype=self.dtype, name="&values_array")
         n = ffi_new("GrB_Index*")
-        scalar = Scalar(n, _INDEX, name="s_nvals", empty=True)
+        scalar = Scalar(n, _INDEX, name="s_nvals", is_cscalar=True, empty=True)
         scalar.value = nvals
         call(
             f"GrB_Vector_extractTuples_{self.dtype.name}", [indices, values, _Pointer(scalar), self]
@@ -258,7 +284,10 @@ class Vector(BaseType):
 
         indices = _CArray(indices)
         values = _CArray(values, self.dtype)
-        call(f"GrB_Vector_build_{self.dtype.name}", [self, indices, values, _CScalar(n), dup_op])
+        call(
+            f"GrB_Vector_build_{self.dtype.name}",
+            [self, indices, values, _as_scalar(n, _INDEX, is_cscalar=True), dup_op],
+        )
 
         # Check for duplicates when dup_op was not provided
         if not dup_op_given and self._nvals < n:
@@ -290,10 +319,7 @@ class Vector(BaseType):
         and make it safe to use as input parameters on multiple threads.
         """
         # TODO: expose COMPLETE or MATERIALIZE options to the user
-        call(
-            "GrB_Vector_wait",
-            [self, Scalar.from_value(lib.GrB_MATERIALIZE, name="GrB_MATERIALIZE")],
-        )
+        call("GrB_Vector_wait", [self, _MATERIALIZE])
 
     @classmethod
     def new(cls, dtype, size=0, *, name=None):
@@ -304,9 +330,9 @@ class Vector(BaseType):
         new_vector = ffi_new("GrB_Vector*")
         dtype = lookup_dtype(dtype)
         rv = cls(new_vector, dtype, name=name)
-        size = _CScalar(size)
+        size = _as_scalar(size, _INDEX, is_cscalar=True)
         call("GrB_Vector_new", [_Pointer(rv), dtype, size])
-        rv._size = size.scalar.value
+        rv._size = size.value
         return rv
 
     @classmethod
@@ -417,6 +443,70 @@ class Vector(BaseType):
             expr.new(name="")  # incompatible shape; raise now
         return expr
 
+    def ewise_union(self, other, op, left_default, right_default):
+        """
+        GxB_Vector_eWiseUnion
+
+        This is similar to `ewise_add` in that result will contain the union of
+        indices from both Vectors.  Unlike `ewise_add`, this will use
+        ``left_default`` for the left value when there is a value on the right
+        but not the left, and ``right_default`` for the right value when there
+        is a value on the left but not the right.
+
+        ``op`` should be a BinaryOp or Monoid.
+        """
+        # SS, SuiteSparse-specific: eWiseUnion
+        method_name = "ewise_union"
+        other = self._expect_type(other, Vector, within=method_name, argname="other", op=op)
+        if type(left_default) is not Scalar:
+            try:
+                left = Scalar.from_value(
+                    left_default, is_cscalar=False, name=""  # pragma: is_grbscalar
+                )
+            except TypeError:
+                left = self._expect_type(
+                    left_default,
+                    Scalar,
+                    within=method_name,
+                    keyword_name="left_default",
+                    extra_message="Literal scalars also accepted.",
+                    op=op,
+                )
+        else:
+            left = _as_scalar(left_default, is_cscalar=False)  # pragma: is_grbscalar
+        if type(right_default) is not Scalar:
+            try:
+                right = Scalar.from_value(
+                    right_default, is_cscalar=False, name=""  # pragma: is_grbscalar
+                )
+            except TypeError:
+                right = self._expect_type(
+                    right_default,
+                    Scalar,
+                    within=method_name,
+                    keyword_name="right_default",
+                    extra_message="Literal scalars also accepted.",
+                    op=op,
+                )
+        else:
+            right = _as_scalar(right_default, is_cscalar=False)  # pragma: is_grbscalar
+        scalar_dtype = unify(left.dtype, right.dtype)
+        nonscalar_dtype = unify(self.dtype, other.dtype)
+        op = get_typed_op(op, scalar_dtype, nonscalar_dtype, is_left_scalar=True, kind="binary")
+        self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
+        if op.opclass == "Monoid":
+            op = op.binaryop
+        expr = VectorExpression(
+            method_name,
+            "GxB_Vector_eWiseUnion",
+            [self, left, other, right],
+            op=op,
+            expr_repr="{0.name}.{method_name}({2.name}, {op}, {1._expr_name}, {3._expr_name})",
+        )
+        if self._size != other._size:
+            expr.new(name="")  # incompatible shape; raise now
+        return expr
+
     def vxm(self, other, op=semiring.plus_times):
         """
         GrB_vxm
@@ -469,7 +559,7 @@ class Vector(BaseType):
         elif right is None:
             if type(left) is not Scalar:
                 try:
-                    left = Scalar.from_value(left, name="")
+                    left = Scalar.from_value(left, is_cscalar=None, name="")
                 except TypeError:
                     left = self._expect_type(
                         left,
@@ -490,13 +580,16 @@ class Vector(BaseType):
                     argname="op",
                     extra_message=extra_message,
                 )
-            cfunc_name = f"GrB_Vector_apply_BinaryOp1st_{left.dtype}"
-            args = [_CScalar(left), self]
-            expr_repr = "{1.name}.apply({op}, left={0})"
+            if left._is_cscalar:
+                cfunc_name = f"GrB_Vector_apply_BinaryOp1st_{left.dtype}"
+            else:
+                cfunc_name = "GrB_Vector_apply_BinaryOp1st_Scalar"
+            args = [left, self]
+            expr_repr = "{1.name}.apply({op}, left={0._expr_name})"
         elif left is None:
             if type(right) is not Scalar:
                 try:
-                    right = Scalar.from_value(right, name="")
+                    right = Scalar.from_value(right, is_cscalar=None, name="")
                 except TypeError:
                     right = self._expect_type(
                         right,
@@ -517,9 +610,12 @@ class Vector(BaseType):
                     argname="op",
                     extra_message=extra_message,
                 )
-            cfunc_name = f"GrB_Vector_apply_BinaryOp2nd_{right.dtype}"
-            args = [self, _CScalar(right)]
-            expr_repr = "{0.name}.apply({op}, right={1})"
+            if right._is_cscalar:
+                cfunc_name = f"GrB_Vector_apply_BinaryOp2nd_{right.dtype}"
+            else:
+                cfunc_name = "GrB_Vector_apply_BinaryOp2nd_Scalar"
+            args = [self, right]
+            expr_repr = "{0.name}.apply({op}, right={1._expr_name})"
         else:
             raise TypeError("Cannot provide both `left` and `right` to apply")
         return VectorExpression(
@@ -531,11 +627,14 @@ class Vector(BaseType):
             size=self._size,
         )
 
-    def reduce(self, op=monoid.plus):
+    def reduce(self, op=monoid.plus, *, allow_empty=True):
         """
         GrB_Vector_reduce
         Reduce all values into a scalar
         Default op is monoid.lor for boolean and monoid.plus otherwise
+
+        For empty Vector objects, the result will be the monoid identity if
+        `allow_empty` is False or empty Scalar if `allow_empty` is True.
         """
         method_name = "reduce"
         op = get_typed_op(op, self.dtype, kind="binary")
@@ -543,11 +642,19 @@ class Vector(BaseType):
             op = op.monoid
         else:
             self._expect_op(op, ("Monoid", "Aggregator"), within=method_name, argname="op")
+        if not allow_empty and op.opclass == "Aggregator" and op.parent._monoid is None:
+            # But we still kindly allow it if it's a monoid-only aggregator such as sum
+            raise ValueError("allow_empty=False not allowed when using Aggregators")
+        if allow_empty:
+            cfunc_name = "GrB_Vector_reduce_Monoid_Scalar"
+        else:
+            cfunc_name = "GrB_Vector_reduce_{output_dtype}"
         return ScalarExpression(
             method_name,
-            "GrB_Vector_reduce_{output_dtype}",
+            cfunc_name,
             [self],
             op=op,  # to be determined later
+            is_cscalar=not allow_empty,
         )
 
     # Unofficial methods
@@ -568,6 +675,7 @@ class Vector(BaseType):
             "GrB_vxm",
             [self, other._as_matrix()],
             op=op,
+            is_cscalar=False,
         )
         if self._size != other._size:
             expr.new(name="")  # incompatible shape; raise now
@@ -604,28 +712,32 @@ class Vector(BaseType):
     ##################################
     # Extract and Assign index methods
     ##################################
-    def _extract_element(self, resolved_indexes, dtype=None, name="s_extract"):
+    def _extract_element(self, resolved_indexes, dtype=None, *, is_cscalar, name=None, result=None):
         if dtype is None:
             dtype = self.dtype
         else:
             dtype = lookup_dtype(dtype)
         idx = resolved_indexes.indices[0]
-        result = Scalar.new(dtype, name=name)
-        if (
-            call(f"GrB_Vector_extractElement_{dtype}", [_Pointer(result), self, idx.index])
-            is not NoValue
-        ):
-            result._is_empty = False
+        if result is None:
+            result = Scalar.new(dtype, is_cscalar=is_cscalar, name=name)
+        if is_cscalar:
+            if (
+                call(f"GrB_Vector_extractElement_{dtype}", [_Pointer(result), self, idx.index])
+                is not NoValue
+            ):
+                result._empty = False
+        else:
+            call("GrB_Vector_extractElement_Scalar", [result, self, idx.index])
         return result
 
     def _prep_for_extract(self, resolved_indexes, mask=None, is_submask=False):
-        size, index, cscalar = resolved_indexes.indices[0]
+        index = resolved_indexes.indices[0]
         return VectorExpression(
             "__getitem__",
             "GrB_Vector_extract",
-            [self, index, cscalar],
-            expr_repr="{0.name}[[{2} elements]]",
-            size=size,
+            [self, index, index.cscalar],
+            expr_repr="{0.name}[{1._expr_name}]",
+            size=index.size,
             dtype=self.dtype,
         )
 
@@ -633,7 +745,7 @@ class Vector(BaseType):
         idx = resolved_indexes.indices[0]
         if type(value) is not Scalar:
             try:
-                value = Scalar.from_value(value, name="")
+                value = Scalar.from_value(value, is_cscalar=None, name="")
             except TypeError:
                 value = self._expect_type(
                     value,
@@ -642,12 +754,21 @@ class Vector(BaseType):
                     argname="value",
                     extra_message="Literal scalars also accepted.",
                 )
-        # should we cast?
-        call(f"GrB_Vector_setElement_{value.dtype}", [self, _CScalar(value), idx.index])
+        if value._is_cscalar:
+            if value._empty:
+                call("GrB_Vector_removeElement", [self, idx.index])
+                return
+            cfunc_name = f"GrB_Vector_setElement_{value.dtype}"
+        else:
+            cfunc_name = "GrB_Vector_setElement_Scalar"
+        call(cfunc_name, [self, value, idx.index])
 
     def _prep_for_assign(self, resolved_indexes, value, mask=None, is_submask=False):
         method_name = "__setitem__"
-        size, index, cscalar = resolved_indexes.indices[0]
+        idx = resolved_indexes.indices[0]
+        size = idx.size
+        cscalar = idx.cscalar
+        index = idx.index
 
         if output_type(value) is Vector:
             if type(value) is not Vector:
@@ -662,16 +783,16 @@ class Vector(BaseType):
                     raise TypeError("Single element assign does not accept a submask")
                 # v[I](m) << w
                 cfunc_name = "GrB_Vector_subassign"
-                expr_repr = "[[{2} elements]](%s) = {0.name}" % mask.name
+                expr_repr = "[[{2._expr_name} elements]](%s) = {0.name}" % mask.name
             else:
                 # v(m)[I] << w
                 # v[I] << w
                 cfunc_name = "GrB_Vector_assign"
-                expr_repr = "[[{2} elements]] = {0.name}"
+                expr_repr = "[[{2._expr_name} elements]] = {0.name}"
         else:
             if type(value) is not Scalar:
                 try:
-                    value = Scalar.from_value(value, name="")
+                    value = Scalar.from_value(value, is_cscalar=None, name="")
                 except TypeError:
                     value = self._expect_type(
                         value,
@@ -680,22 +801,27 @@ class Vector(BaseType):
                         argname="value",
                         extra_message="Literal scalars also accepted.",
                     )
-            value = _CScalar(value)
             if is_submask:
                 if size is None:
                     # v[i](m) << c
                     raise TypeError("Single element assign does not accept a submask")
                 # v[I](m) << c
-                cfunc_name = f"GrB_Vector_subassign_{value.dtype}"
-                expr_repr = "[[{2} elements]](%s) = {0}" % mask.name
+                if value._is_cscalar:
+                    cfunc_name = f"GrB_Vector_subassign_{value.dtype}"
+                else:
+                    cfunc_name = "GrB_Vector_subassign_Scalar"
+                expr_repr = "[[{2._expr_name} elements]](%s) = {0._expr_name}" % mask.name
             else:
                 # v(m)[I] << c
                 # v[I] << c
                 if size is None:
-                    index = _CArray([index.scalar.value])
-                    cscalar = _CScalar(1)
-                cfunc_name = f"GrB_Vector_assign_{value.dtype}"
-                expr_repr = "[[{2} elements]] = {0}"
+                    index = _CArray([index.value])
+                    cscalar = _as_scalar(1, _INDEX, is_cscalar=True)
+                if value._is_cscalar:
+                    cfunc_name = f"GrB_Vector_assign_{value.dtype}"
+                else:
+                    cfunc_name = "GrB_Vector_assign_Scalar"
+                expr_repr = "[[{2._expr_name} elements]] = {0._expr_name}"
         return VectorExpression(
             method_name,
             cfunc_name,
@@ -809,7 +935,7 @@ class VectorExpression(BaseExpression):
     def shape(self):
         return (self._size,)
 
-    # Paste here from _automethods.py
+    # Begin auto-generated code: Vector
     _get_value = _automethods._get_value
     S = wrapdoc(Vector.S)(property(_automethods.S))
     V = wrapdoc(Vector.V)(property(_automethods.V))
@@ -822,12 +948,14 @@ class VectorExpression(BaseExpression):
     __rand__ = wrapdoc(Vector.__rand__)(property(_automethods.__rand__))
     __rmatmul__ = wrapdoc(Vector.__rmatmul__)(property(_automethods.__rmatmul__))
     __ror__ = wrapdoc(Vector.__ror__)(property(_automethods.__ror__))
+    _as_matrix = wrapdoc(Vector._as_matrix)(property(_automethods._as_matrix))
     _carg = wrapdoc(Vector._carg)(property(_automethods._carg))
     _name_html = wrapdoc(Vector._name_html)(property(_automethods._name_html))
     _nvals = wrapdoc(Vector._nvals)(property(_automethods._nvals))
     apply = wrapdoc(Vector.apply)(property(_automethods.apply))
     ewise_add = wrapdoc(Vector.ewise_add)(property(_automethods.ewise_add))
     ewise_mult = wrapdoc(Vector.ewise_mult)(property(_automethods.ewise_mult))
+    ewise_union = wrapdoc(Vector.ewise_union)(property(_automethods.ewise_union))
     gb_obj = wrapdoc(Vector.gb_obj)(property(_automethods.gb_obj))
     inner = wrapdoc(Vector.inner)(property(_automethods.inner))
     isclose = wrapdoc(Vector.isclose)(property(_automethods.isclose))
@@ -843,8 +971,8 @@ class VectorExpression(BaseExpression):
     vxm = wrapdoc(Vector.vxm)(property(_automethods.vxm))
     wait = wrapdoc(Vector.wait)(property(_automethods.wait))
     # These raise exceptions
-    __array__ = wrapdoc(Vector.__array__)(Vector.__array__)
-    __bool__ = wrapdoc(Vector.__bool__)(Vector.__bool__)
+    __array__ = Vector.__array__
+    __bool__ = Vector.__bool__
     __iadd__ = _automethods.__iadd__
     __iand__ = _automethods.__iand__
     __ifloordiv__ = _automethods.__ifloordiv__
@@ -856,9 +984,80 @@ class VectorExpression(BaseExpression):
     __isub__ = _automethods.__isub__
     __itruediv__ = _automethods.__itruediv__
     __ixor__ = _automethods.__ixor__
+    # End auto-generated code: Vector
+
+
+class VectorIndexExpr(AmbiguousAssignOrExtract):
+    __slots__ = "_size"
+    ndim = 1
+    output_type = Vector
+
+    def __init__(self, parent, resolved_indexes, size):
+        super().__init__(parent, resolved_indexes)
+        self._size = size
+
+    @property
+    def size(self):
+        return self._size
+
+    @property
+    def shape(self):
+        return (self._size,)
+
+    # Begin auto-generated code: Vector
+    _get_value = _automethods._get_value
+    S = wrapdoc(Vector.S)(property(_automethods.S))
+    V = wrapdoc(Vector.V)(property(_automethods.V))
+    __and__ = wrapdoc(Vector.__and__)(property(_automethods.__and__))
+    __contains__ = wrapdoc(Vector.__contains__)(property(_automethods.__contains__))
+    __getitem__ = wrapdoc(Vector.__getitem__)(property(_automethods.__getitem__))
+    __iter__ = wrapdoc(Vector.__iter__)(property(_automethods.__iter__))
+    __matmul__ = wrapdoc(Vector.__matmul__)(property(_automethods.__matmul__))
+    __or__ = wrapdoc(Vector.__or__)(property(_automethods.__or__))
+    __rand__ = wrapdoc(Vector.__rand__)(property(_automethods.__rand__))
+    __rmatmul__ = wrapdoc(Vector.__rmatmul__)(property(_automethods.__rmatmul__))
+    __ror__ = wrapdoc(Vector.__ror__)(property(_automethods.__ror__))
+    _as_matrix = wrapdoc(Vector._as_matrix)(property(_automethods._as_matrix))
+    _carg = wrapdoc(Vector._carg)(property(_automethods._carg))
+    _name_html = wrapdoc(Vector._name_html)(property(_automethods._name_html))
+    _nvals = wrapdoc(Vector._nvals)(property(_automethods._nvals))
+    apply = wrapdoc(Vector.apply)(property(_automethods.apply))
+    ewise_add = wrapdoc(Vector.ewise_add)(property(_automethods.ewise_add))
+    ewise_mult = wrapdoc(Vector.ewise_mult)(property(_automethods.ewise_mult))
+    ewise_union = wrapdoc(Vector.ewise_union)(property(_automethods.ewise_union))
+    gb_obj = wrapdoc(Vector.gb_obj)(property(_automethods.gb_obj))
+    inner = wrapdoc(Vector.inner)(property(_automethods.inner))
+    isclose = wrapdoc(Vector.isclose)(property(_automethods.isclose))
+    isequal = wrapdoc(Vector.isequal)(property(_automethods.isequal))
+    name = wrapdoc(Vector.name)(property(_automethods.name))
+    name = name.setter(_automethods._set_name)
+    nvals = wrapdoc(Vector.nvals)(property(_automethods.nvals))
+    outer = wrapdoc(Vector.outer)(property(_automethods.outer))
+    reduce = wrapdoc(Vector.reduce)(property(_automethods.reduce))
+    ss = wrapdoc(Vector.ss)(property(_automethods.ss))
+    to_pygraphblas = wrapdoc(Vector.to_pygraphblas)(property(_automethods.to_pygraphblas))
+    to_values = wrapdoc(Vector.to_values)(property(_automethods.to_values))
+    vxm = wrapdoc(Vector.vxm)(property(_automethods.vxm))
+    wait = wrapdoc(Vector.wait)(property(_automethods.wait))
+    # These raise exceptions
+    __array__ = Vector.__array__
+    __bool__ = Vector.__bool__
+    __iadd__ = _automethods.__iadd__
+    __iand__ = _automethods.__iand__
+    __ifloordiv__ = _automethods.__ifloordiv__
+    __imatmul__ = _automethods.__imatmul__
+    __imod__ = _automethods.__imod__
+    __imul__ = _automethods.__imul__
+    __ior__ = _automethods.__ior__
+    __ipow__ = _automethods.__ipow__
+    __isub__ = _automethods.__isub__
+    __itruediv__ = _automethods.__itruediv__
+    __ixor__ = _automethods.__ixor__
+    # End auto-generated code: Vector
 
 
 utils._output_types[Vector] = Vector
+utils._output_types[VectorIndexExpr] = Vector
 utils._output_types[VectorExpression] = Vector
 
 # Import matrix to import infix to import _infixmethods, which has side effects
